from __future__ import annotations

import logging
import pathlib
import typing as T

import xarray as xr

from . import _core as core

logger = logging.getLogger(__name__)

# Constants

# https://stackoverflow.com/a/72832981/592289
# Types
SRTM15PVersion = T.Literal["2.6"]
# Constants
SRTM15P: T.Literal["SRTM15+"] = "SRTM15+"
SRTM15P_LATEST_VERSION: SRTM15PVersion = T.get_args(SRTM15PVersion)[-1]


class SRTM15PRecord(T.TypedDict):
    url: str
    filename: str
    hash: str


def srtm15p(
    version: SRTM15PVersion = SRTM15P_LATEST_VERSION,
    *,
    registry_url: str | None = None,
    as_paths: bool = False,
) -> list[core.CachedPaths]:
    """
    Return the path to a SRTM15+ dataset, downloading the dataset if necessary.

    Parameters:
        version: The SRTM15+ version to use. Defaults to the latest version available.
        registry_url: The URL to a registry that provides the dataset metadata.
            If None, the default registry is used.

    Returns:
        str: The path of the requested SRTM15+ dataset in the local cache.

    """
    core.enforce_literals(srtm15p)
    registry = core.load_registry(registry_url=registry_url)
    record = registry[SRTM15P][version]
    cache_dir = core.get_cache_path() / SRTM15P / version
    file_path = cache_dir / record["filename"]
    if not file_path.exists():
        cache_dir.mkdir(parents=True, exist_ok=True)
        core.download(record["url"], file_path)
    core.check_hash(file_path, record["hash"])
    if as_paths:
        return [pathlib.Path(file_path)]
    else:
        return [str(file_path)]


def srtm15p_ds(
    version: SRTM15PVersion = SRTM15P_LATEST_VERSION,
    *,
    registry_url: str | None = None,
    **kwargs: T.Any,
) -> xr.Dataset:
    path = srtm15p(
        version=version,
        registry_url=registry_url,
    )[0]
    if "engine" not in kwargs:
        kwargs["engine"] = "h5netcdf"
    ds = xr.open_dataset(path, **kwargs)
    return ds



from __future__ import annotations

import logging
import pathlib
import typing as T

import xarray as xr

from . import _core as core

logger = logging.getLogger(__name__)

# Constants

# https://stackoverflow.com/a/72832981/592289
# Types
GEBCODatasets = T.Literal["ice", "sub_ice"]
GEBCOVersion = T.Literal["2022", "2023", "2024"]
# Constants
GEBCO: T.Literal["GEBCO"] = "GEBCO"
GEBCO_LATEST_VERSION: GEBCOVersion = T.get_args(GEBCOVersion)[-1]


class GEBCORecord(T.TypedDict):
    url: str
    archive: str
    filename: str
    hash: str


def gebco(
    dataset: GEBCODatasets,
    version: GEBCOVersion = GEBCO_LATEST_VERSION,
    *,
    registry_url: str | None = None,
    as_paths: bool = False,
) -> list[core.CachedPaths]:
    """
    Return the path to a GEBCO dataset, downloading the dataset if necessary.

    Parameters:
        dataset: The name of the GEBCO dataset. Possible values: `ice`, `sub_ice`.
        version: The GEBCO version to use. Defaults to the latest version available.
        registry_url: The URL to a registry that provides the dataset metadata.
            If None, the default registry is used.

    Returns:
        str: The path of the requested GEBCO dataset in the local cache.

    """
    core.enforce_literals(gebco)
    registry = core.load_registry(registry_url=registry_url)
    version_str = str(version)
    record = registry[GEBCO][version_str][dataset]
    cache_dir = core.get_cache_path() / GEBCO / version_str / dataset
    file_path = cache_dir / record["filename"]
    if not file_path.exists():
        cache_dir.mkdir(parents=True, exist_ok=True)
        archive_path = cache_dir / record["archive"]
        core.download(record["url"], archive_path)
        core.extract_zip(archive_path, record["filename"], cache_dir)
        core.lenient_remove(archive_path)
    core.check_hash(file_path, record["hash"])
    if as_paths:
        return [pathlib.Path(file_path)]
    else:
        return [str(file_path)]


def gebco_ds(
    dataset: GEBCODatasets,
    version: GEBCOVersion = GEBCO_LATEST_VERSION,
    *,
    registry_url: str | None = None,
    **kwargs: T.Any,
) -> xr.Dataset:
    path = gebco(
        dataset=dataset,
        version=version,
        registry_url=registry_url,
    )[0]
    if "engine" not in kwargs:
        kwargs["engine"] = "h5netcdf"
    ds = xr.open_dataset(path, **kwargs)
    return ds



import importlib.metadata

from ._emodnet import emodnet
from ._etopo import etopo
from ._etopo import etopo_ds
from ._gebco import gebco
from ._gebco import gebco_ds
from ._gshhg import gshhg
from ._gshhg import gshhg_df
from ._osm import osm
from ._osm import osm_df
from ._rtopo import rtopo
from ._rtopo import rtopo_ds
from ._srtm15p import srtm15p
from ._srtm15p import srtm15p_ds
from ._utm import utm_df

__version__ = importlib.metadata.version(__name__)


__all__: list[str] = [
    "emodnet",
    "etopo",
    "etopo_ds",
    "gebco",
    "gebco_ds",
    "gshhg",
    "gshhg_df",
    "osm",
    "osm_df",
    "rtopo",
    "rtopo_ds",
    "srtm15p",
    "srtm15p_ds",
    "utm_df",
    "__version__",
]



from __future__ import annotations

import dataclasses
import functools

import geopandas as gpd
import shapely
from shapely import box

# https://en.wikipedia.org/wiki/Universal_Transverse_Mercator_coordinate_system#
# This is essentially a static dataset but it doesn't seem to be readily downloadable.
# The only sourcews I found were from ArcGIS and they needed to be converted
# Recreating is not that hard, so this is what we do here.
# We avoid the need to host it, too.


@dataclasses.dataclass
class Tile:
    zone: int
    row: str
    code: str
    geometry: shapely.Polygon


@functools.cache
def utm_df() -> gpd.GeoDataFrame:
    tiles: list[Tile] = []

    # South Pole
    tiles.extend(
        [
            Tile(0, "A", "esri:102021", box(-180, -90, 0, -80)),
            Tile(0, "B", "esri:102021", box(0, -90, 180, -80)),
        ],
    )

    for zone in range(1, 61):
        # southern hemisphere
        for i, row in enumerate("CDEFGHJKLM"):
            polygon = box((zone - 1) * 6 - 180, -80 + i * 8, zone * 6 - 180, -80 + (i + 1) * 8)
            tiles.append(Tile(zone, row, f"epsg:327{zone:02d}", polygon))
        # northern hemisphere
        for i, row in enumerate("NPQRSTUVW"):
            polygon = box((zone - 1) * 6 - 180, i * 8, zone * 6 - 180, (i + 1) * 8)
            tiles.append(Tile(zone, row, f"epsg:326{zone:02d}", polygon))
        # row X
        if zone in (32, 34, 36):
            continue
        polygon = box((zone - 1) * 6 - 180, 72, zone * 6 - 180, 84)
        tiles.append(Tile(zone, "X", f"epsg:326{zone:02d}", polygon))

    # North Pole
    tiles.extend(
        [
            Tile(0, "X", "esri:102018", box(-180, 84, 0, 90)),
            Tile(0, "Y", "esri:102018", box(0, 84, 180, 90)),
        ],
    )

    utm = gpd.GeoDataFrame(tiles, crs=4326)

    # Apply Norway exceptions
    utm.loc[(utm.zone == 31) & (utm.row == "V"), "geometry"] = shapely.box(0, 56, 3, 64)  # type: ignore[call-overload]  # noqa: PLR2004
    utm.loc[(utm.zone == 32) & (utm.row == "V"), "geometry"] = shapely.box(3, 56, 12, 64)  # type: ignore[call-overload]  # noqa: PLR2004
    utm.loc[(utm.zone == 31) & (utm.row == "X"), "geometry"] = shapely.box(0, 72, 9, 84)  # type: ignore[call-overload]  # noqa: PLR2004
    utm.loc[(utm.zone == 33) & (utm.row == "X"), "geometry"] = shapely.box(9, 72, 21, 84)  # type: ignore[call-overload]  # noqa: PLR2004
    utm.loc[(utm.zone == 35) & (utm.row == "X"), "geometry"] = shapely.box(21, 72, 33, 84)  # type: ignore[call-overload]  # noqa: PLR2004
    utm.loc[(utm.zone == 37) & (utm.row == "X"), "geometry"] = shapely.box(33, 72, 42, 84)  # type: ignore[call-overload]  # noqa: PLR2004

    return utm



from __future__ import annotations

import logging
import typing as T

from . import _core as core

logger = logging.getLogger(__name__)


# https://stackoverflow.com/a/72832981/592289
# Types
ETopoDataset = T.Literal["bedrock", "ice_surface"]
# Constants
ETOPO1: T.Literal["ETOPO1"] = "ETOPO1"


def get_etopo_filename(dataset: ETopoDataset) -> str:
    filename = f"ETOPO1_{dataset}_g_gdal.nc"
    return filename


def etopo1(
    dataset: ETopoDataset,
    *,
    registry_url: str | None = None,
) -> str:
    core.enforce_literals(etopo1)
    registry = core.load_registry(registry_url=registry_url)
    record = registry[ETOPO1][dataset]
    cache_dir = core.get_cache_path() / ETOPO1
    filename = str(record["filename"])
    path = cache_dir / filename
    if not path.exists():
        cache_dir.mkdir(parents=True, exist_ok=True)
        url = str(record["url"])
        archive_path = cache_dir / record["archive"]
        # core.download(url, archive_path)
        core.extract_gzip(archive_path, path)
        # core.lenient_remove(archive_path)
    core.check_hash(path, str(record["hash"]))
    return str(path)



from __future__ import annotations

import gzip
import importlib.resources
import inspect
import json
import logging
import os
import pathlib
import shutil
import sys
import typing as T
import zipfile

import httpx
import platformdirs
import stamina
import xxhash
import zstandard
from tqdm.auto import tqdm

logger = logging.getLogger(__name__)


# Types
CachedPaths: T.TypeAlias = str | pathlib.Path


# https://stackoverflow.com/a/72832981/592289
# 1. Use `eval_str=True` in order to allow `from __future__ import annotations`
# 2. Change the order somewhat in order to make it faster
def enforce_literals(function: T.Callable[..., T.Any]) -> None:
    kwargs = sys._getframe(1).f_locals
    for name, type_ in inspect.get_annotations(function, eval_str=True).items():
        if T.get_origin(type_) is T.Literal:
            value = kwargs.get(name)
            options = set(T.get_args(type_))
            if name in kwargs and value not in options:
                raise ValueError(f"Wrong value for '{name}': '{value}' is not in {options}")


def resolve_httpx_client(
    client: httpx.Client | None = None,
    timeout: float = 20,
    read: float = 30,
) -> httpx.Client:
    if client is None:
        client = httpx.Client(
            timeout=httpx.Timeout(timeout=timeout, read=read),
        )
    return client


@stamina.retry(on=httpx.HTTPError, attempts=3)
def download(
    url: str,
    filename: os.PathLike[str] | str,
    client: httpx.Client | None = None,
) -> None:
    client = resolve_httpx_client(client=client)
    with client.stream("GET", url) as response:
        response.raise_for_status()
        with open(filename, "wb") as fd:
            total = int(response.headers["Content-Length"])
            tqdm_params = {
                "desc": url,
                "total": total,
                "miniters": 1,
                "unit": "B",
                "unit_scale": True,
                "unit_divisor": 1024,
            }
            with tqdm(**tqdm_params) as progress:
                downloaded = response.num_bytes_downloaded
                for chunk in response.iter_bytes():
                    fd.write(chunk)
                    progress.update(response.num_bytes_downloaded - downloaded)
                    downloaded = response.num_bytes_downloaded


def extract_zip(archive: os.PathLike[str] | str, filename: str, target_dir: os.PathLike[str] | str) -> None:
    logger.debug(f"Extracting {filename} to: {target_dir}")
    with zipfile.ZipFile(archive, "r") as zip_ref:
        zip_ref.extract(member=filename, path=target_dir)


def extract_gzip(archive: os.PathLike[str] | str, target: os.PathLike[str] | str) -> None:
    logger.debug(f"Extracting {archive} to: {target}")
    with gzip.open(archive, "rb") as f_in:
        with open(target, "wb") as f_out:
            shutil.copyfileobj(f_in, f_out)


def extract_zstd(archive: os.PathLike[str] | str, target: os.PathLike[str] | str) -> None:
    logger.debug(f"Extracting {archive} to: {target}")
    dctx = zstandard.ZstdDecompressor()
    with open(archive, "rb") as ifh, open(target, "wb") as ofh:
        dctx.copy_stream(ifh, ofh)


def hash_file(path: os.PathLike[str] | str, chunksize: int = 2**20) -> str:
    hasher = xxhash.xxh128()
    with open(path, "rb") as fd:
        for chunk in iter(lambda: fd.read(chunksize), b""):
            hasher.update(chunk)
    return hasher.hexdigest()


def check_hash(path: os.PathLike[str] | str, expected_hash: str) -> None:
    logger.debug(f"Checking hash of: {path}")
    current_hash = hash_file(path)
    if current_hash != expected_hash:
        raise ValueError(f"hash mismatch: {current_hash} != {expected_hash}")


def get_cache_path() -> pathlib.Path:
    cache = os.environ.get(
        "SEAREPORT_DATA_DIR",
        platformdirs.user_cache_dir("seareport_data"),
    )
    cache_path = pathlib.Path(cache)
    cache_path.mkdir(parents=True, exist_ok=True)
    return cache_path


def lenient_remove(path: os.PathLike[str] | str) -> None:
    if os.path.exists(path):
        try:
            os.remove(path)
        except Exception:
            logger.exception("Failed to remove: %s", path)


def lenient_remove_tree(path: os.PathLike[str] | str) -> None:
    try:
        shutil.rmtree(path)
    except Exception:
        logger.exception("Failed to remove: %s", path)


def load_registry(registry_url: str | None = None) -> dict[str, dict[str, dict[str, T.Any]]]:
    if registry_url:
        registry: dict[str, T.Any] = httpx.get(registry_url, timeout=30).json()
    else:
        registry = json.load(importlib.resources.open_text("seareport_data", "registry.json"))
    return registry



from __future__ import annotations

import logging
import pathlib
import typing as T

import geopandas as gpd

from . import _core as core

logger = logging.getLogger(__name__)


# https://stackoverflow.com/a/72832981/592289
# Types
OSMDataset = T.Literal["land"]
OSMVersion = T.Literal["2025-01"]
# Constants
OSM: T.Literal["OSM"] = "OSM"
OSM_LATEST_VERSION: OSMVersion = T.get_args(OSMVersion)[-1]


def get_osm_filename(dataset: OSMDataset) -> str:
    filename = f"osm_{dataset}_complete_4326.sqlite"
    return filename


def osm(
    dataset: OSMDataset = "land",
    version: OSMVersion = OSM_LATEST_VERSION,
    *,
    registry_url: str | None = None,
    as_paths: bool = False,
) -> list[core.CachedPaths]:
    core.enforce_literals(osm)
    registry = core.load_registry(registry_url=registry_url)
    record = registry[OSM][str(version)][dataset]
    logger.debug("Record: %s", record)
    cache_dir = core.get_cache_path() / OSM / version
    filename = str(record["filename"])
    path = cache_dir / filename
    if not path.exists():
        cache_dir.mkdir(parents=True, exist_ok=True)
        archive_path = cache_dir / record["archive"]
        url = str(record["url"])
        core.download(url, archive_path)
        core.extract_zstd(archive_path, path)
        core.lenient_remove(archive_path)
    core.check_hash(path, str(record["hash"]))
    if as_paths:
        return [pathlib.Path(path)]
    else:
        return [str(path)]


def osm_df(
    dataset: OSMDataset = "land",
    version: OSMVersion = OSM_LATEST_VERSION,
    *,
    registry_url: str | None = None,
    **kwargs: T.Any,
) -> gpd.GeoDataFrame:
    path = osm(
        dataset=dataset,
        version=version,
        registry_url=registry_url,
    )[0]
    gdf: gpd.GeoDataFrame = gpd.read_file(path, engine="pyogrio", spatialite=True, **kwargs)
    return gdf



from __future__ import annotations

import logging
import pathlib
import typing as T

import geopandas as gpd

from . import _core as core

logger = logging.getLogger(__name__)

# https://stackoverflow.com/a/72832981/592289
# Types
GSHHGVersion = T.Literal["2.3.7.1"]
GSHHGResolution = T.Literal[
    "c",
    "l",
    "i",
    "h",
    "f",
    "C",
    "L",
    "I",
    "H",
    "F",
    "crude",
    "low",
    "intermediate",
    "high",
    "full",
]
GSHHGShoreline = T.Literal["5", "6"]
# Constants
GSHHG: T.Literal["GSHHG"] = "GSHHG"
CRUDE: T.Literal["crude"] = "crude"
LOW: T.Literal["low"] = "low"
INTERMEDIATE: T.Literal["intermediate"] = "intermediate"
HIGH: T.Literal["high"] = "high"
FULL: T.Literal["full"] = "full"
GSHHG_LATEST_VERSION: GSHHGVersion = sorted(T.get_args(GSHHGVersion))[-1]


class GSHHGRecord(T.TypedDict):
    doi: str
    base_url: str
    hashes: dict[str, str]


SHORT_TO_LONG_GSHHG_RESOLUTIONS = {
    "c": CRUDE,
    "l": LOW,
    "i": INTERMEDIATE,
    "h": HIGH,
    "f": FULL,
}


def get_gshhg_filename(
    resolution: GSHHGResolution,
    shoreline: GSHHGShoreline,
) -> str:
    short_resolution = resolution[0].lower()
    long_resolution = SHORT_TO_LONG_GSHHG_RESOLUTIONS[short_resolution]
    return f"gshhg_{long_resolution}_l{shoreline}.gpkg"


def gshhg(
    resolution: GSHHGResolution,
    shoreline: GSHHGShoreline,
    version: GSHHGVersion = GSHHG_LATEST_VERSION,
    *,
    registry_url: str | None = None,
    as_paths: bool = False,
) -> list[core.CachedPaths]:
    core.enforce_literals(gshhg)
    registry = core.load_registry(registry_url=registry_url)
    record = registry[GSHHG][version]
    cache_dir = core.get_cache_path() / GSHHG / version
    filename = get_gshhg_filename(resolution=resolution, shoreline=shoreline)
    path = cache_dir / filename
    if not path.exists():
        cache_dir.mkdir(parents=True, exist_ok=True)
        url = record["base_url"] + filename
        core.download(url, path)
    core.check_hash(path, record["hashes"][filename])
    if as_paths:
        return [pathlib.Path(path)]
    else:
        return [str(path)]


def gshhg_df(
    resolution: GSHHGResolution,
    shoreline: GSHHGShoreline,
    version: GSHHGVersion = GSHHG_LATEST_VERSION,
    *,
    registry_url: str | None = None,
    **kwargs: T.Any,
) -> gpd.GeoDataFrame:
    path = gshhg(
        resolution=resolution,
        shoreline=shoreline,
        version=version,
        registry_url=registry_url,
    )[0]
    gdf: gpd.GeoDataFrame = gpd.read_file(path, **kwargs)
    return gdf



from __future__ import annotations

import logging
import pathlib
import typing as T

import xarray as xr

from . import _core as core

logger = logging.getLogger(__name__)


# https://stackoverflow.com/a/72832981/592289
# Types
RTopoVersion = T.Literal["2.0.4"]
RTopoDataset = T.Literal["bedrock", "ice_base", "ice_thickness", "surface_elevation"]
# Constants
RTOPO: T.Literal["RTOPO"] = "RTOPO"
RTOPO_LATEST_VERSION: RTopoVersion = sorted(T.get_args(RTopoVersion))[-1]


def get_rtopo_filename(dataset: RTopoDataset, version: RTopoVersion) -> str:
    dataset_name = str(dataset)
    if dataset in ("bedrock", "ice_base"):
        dataset_name += "_topography"
    filename = f"RTopo-{version}_30sec_{dataset_name}.nc"
    return filename


def rtopo(
    dataset: RTopoDataset,
    version: RTopoVersion = RTOPO_LATEST_VERSION,
    *,
    registry_url: str | None = None,
    as_paths: bool = False,
) -> list[core.CachedPaths]:
    core.enforce_literals(rtopo)
    registry = core.load_registry(registry_url=registry_url)
    record = registry[RTOPO][version]
    cache_dir = core.get_cache_path() / RTOPO / version
    filename = get_rtopo_filename(dataset=dataset, version=version)
    path = cache_dir / filename
    if not path.exists():
        cache_dir.mkdir(parents=True, exist_ok=True)
        url = record["base_url"] + filename
        core.download(url, path)
    core.check_hash(path, record["hashes"][filename])
    if as_paths:
        return [pathlib.Path(path)]
    else:
        return [str(path)]


def rtopo_ds(
    dataset: RTopoDataset,
    version: RTopoVersion = RTOPO_LATEST_VERSION,
    *,
    registry_url: str | None = None,
    normalize: bool = True,
    **kwargs: T.Any,
) -> xr.Dataset:
    path = rtopo(
        dataset=dataset,
        version=version,
        registry_url=registry_url,
    )[0]
    if "engine" in kwargs and kwargs["engine"] == "h5netcdf":
        raise ValueError("RTopo is in netcdf classic format, which is not supported by `h5netcdf` engine.")
    ds = xr.open_dataset(path, **kwargs)
    if normalize:
        ds = ds.swap_dims({"londim": "lon", "latdim": "lat"})
        ds = ds.rename_vars({next(iter(ds.data_vars)): "z"})
        ds.lon.attrs.update(
            {
                "name": "lon",
                "long_name": "longitude coordinate",
                "units": "degrees_east",
                "standard_name": "longitude",
            },
        )
        ds.lat.attrs.update(
            {
                "name": "lat",
                "long_name": "latitude coordinate",
                "units": "degrees_north",
                "standard_name": "latitude",
            },
        )
        if "title" in ds.z.attrs:
            ds.z.attrs["title"] = f"RTopo {ds.z.attrs['title']}"
        else:
            ds.z.attrs["title"] = "RTopo - ice_thickness"
    return ds



from __future__ import annotations

import logging
import pathlib
import typing as T

import xarray as xr

from . import _core as core

logger = logging.getLogger(__name__)


# https://stackoverflow.com/a/72832981/592289
# Types
ETopoDataset = T.Literal["bedrock", "surface", "geoid"]
ETopoVersion = T.Literal["2022"]
ETopoResolution = T.Literal["30sec", "60sec"]
# Constants
ETOPO: T.Literal["ETOPO"] = "ETOPO"
ETOPO_LATEST_VERSION: ETopoVersion = T.get_args(ETopoVersion)[-1]


def get_etopo_filename(dataset: ETopoDataset) -> str:
    filename = f"ETOPO1_{dataset}_g_gdal.nc"
    return filename


def etopo(
    dataset: ETopoDataset,
    resolution: ETopoResolution = "30sec",
    version: ETopoVersion = ETOPO_LATEST_VERSION,
    *,
    registry_url: str | None = None,
    as_paths: bool = False,
) -> list[core.CachedPaths]:
    core.enforce_literals(etopo)
    registry = core.load_registry(registry_url=registry_url)
    record = registry[ETOPO][str(version)][resolution][dataset]
    cache_dir = core.get_cache_path() / ETOPO / version
    filename = str(record["filename"])
    path = cache_dir / filename
    if not path.exists():
        cache_dir.mkdir(parents=True, exist_ok=True)
        url = str(record["url"])
        core.download(url, path)
    core.check_hash(path, str(record["hash"]))
    if as_paths:
        return [pathlib.Path(path)]
    else:
        return [str(path)]


def etopo_ds(
    dataset: ETopoDataset,
    resolution: ETopoResolution = "30sec",
    version: ETopoVersion = ETOPO_LATEST_VERSION,
    *,
    registry_url: str | None = None,
    **kwargs: T.Any,
) -> xr.Dataset:
    path = etopo(
        dataset=dataset,
        resolution=resolution,
        version=version,
        registry_url=registry_url,
    )[0]
    if "engine" not in kwargs:
        kwargs["engine"] = "h5netcdf"
    ds = xr.open_dataset(path, **kwargs)
    return ds



from __future__ import annotations

import logging
import typing as T

from . import _core as core

logger = logging.getLogger(__name__)


# https://stackoverflow.com/a/72832981/592289
# Types
EMODnetVersion = T.Literal["2022"]
# Constants
EMODNET: T.Literal["EMODnet"] = "EMODnet"
EMODNET_LATEST_VERSION: EMODnetVersion = T.get_args(EMODnetVersion)[-1]


def emodnet(
    version: EMODnetVersion = EMODNET_LATEST_VERSION,
    *,
    registry_url: str | None = None,
    as_paths: bool = False,
) -> list[core.CachedPaths]:
    core.enforce_literals(emodnet)
    registry = core.load_registry(registry_url=registry_url)
    record = registry[EMODNET][str(version)]
    base_url = T.cast(str, record["base_url"])
    cache_dir = core.get_cache_path() / EMODNET / version
    paths: list[core.CachedPaths] = []
    for filename, expected_hash in record["hashes"].items():
        assert isinstance(filename, str)
        assert isinstance(expected_hash, str)
        archive = cache_dir / f"{filename}.zip"
        path = cache_dir / filename
        if not path.exists():
            cache_dir.mkdir(parents=True, exist_ok=True)
            url = f"{base_url}{filename}.zip"
            core.download(url, archive)
            core.extract_zip(archive=archive, filename=filename, target_dir=cache_dir)
            core.check_hash(path, expected_hash)
            core.lenient_remove(archive)
        paths.append(path)
    if as_paths:
        return paths
    else:
        return [str(path) for path in paths]


# def emodnet_ds(
#     version: EMODnetVersion = EMODNET_LATEST_VERSION,
#     *,
#     registry_url: str | None = None,
#     **kwargs: T.Any,
# ) -> xr.Dataset:
#     paths = emodnet(
#         version=version,
#         registry_url=registry_url,
#     )
#     if "engine" not in kwargs:
#         kwargs["engine"] = "h5netcdf"
#     ds = xr.open_dataset(paths, **kwargs)
#     return ds
